@@ -44,7 +44,9 @@ class OctileGraph:
         for df in [nodes, edges]:
             for key in list(df.columns):
                 if not np.issctype(df[key].dtype):
-                    if df[key].concrete_type in [list, tuple, np.ndarray]:
+                    if issubclass(df[key].concrete_type, (list,
+                                                          tuple,
+                                                          np.ndarray)):
                         inner_type = common_min_type.of_types(
                             [x.dtype if isinstance(x, np.ndarray)
                              else common_min_type.of_values(x)
@@ -142,6 +144,9 @@ class OctileGraph:
         lf[:] = j - j % 8
 
         perm = np.lexsort(indices, axis=0)
+        # Do not delete this line even though it looks useless. It does change
+        # i, j, up, lf
+        indices[:, :] = indices[:, perm]
         self.edges_aos = umempty(nnz * 2, edge_t)
         self.edges_aos[:] = edges_aos[perm % nnz]  # mod nnz due to symmetry
 
